#ifndef __MESHSIMP_H
#define __MESHSIMP_H

#include <stdio.h>
#include <math.h>

#include "DEM.h"
#include "GeoPrim.h"
#include "Global.h"
#include "VRML.h"

static const int dx[4] = {1, 1, 0, 1};
static const int dy[4] = {0, 1, 1, -1};

class BaseMeshSimplifier
{
public:
  class OutFile *vrmlfile;
  int notRemovedCounter;
  COOR dim;
  ELEV **data;
  // the arrays of not removed nodes' coordinates
  COOR *datax;
  COOR *dataz;
  
  BaseMeshSimplifier(ELEV **demData, OutFile *ivfile, int aDim);
  ~BaseMeshSimplifier();
  
  virtual void simplifyMesh(ELEV thresh);
  void outputPoint(COOR i, COOR j);
  int getNotRemovedCounter();
};
  
class MeshSimplifier:public BaseMeshSimplifier
{
  COOR ***adddata;

public:
  MeshSimplifier(ELEV **demData, OutFile *ivfile, int aDim);
  ~MeshSimplifier();
  
  virtual void simplifyMesh(ELEV thresh);
  void prepareAdditionalData();
  void removeAdditionalData();
};

class SecDerSimplifier: public BaseMeshSimplifier
{
  ELEV **derData1;
  ELEV **secDerData1;
  ELEV **derData2;
  ELEV **secDerData2;
  ELEV **derData3;
  ELEV **secDerData3;
  ELEV **derData4;
  ELEV **secDerData4;
  
public:
  SecDerSimplifier(ELEV **demData, OutFile *ivfile, int aDim);
  ~SecDerSimplifier();

  virtual void simplifyMesh(ELEV thresh);
  void prepareDerData();
  void removeDerData();
  
};

#endif
